#!/bin/bash

# Здесь не стоит делать set -eu
# Здесь не стоит менять названия цепочек
# Переменная forward для обеспечения совместимости кода между CR7 и CR8

. /etc/rc.d/init.d/functions
. /usr/local/Reductor/etc/const
. $CONFIG

declare -a rules
rules=(
	"-p tcp -j REJECT --reject-with tcp-reset"
	"-p udp -j REJECT --reject-with icmp-port-unreachable"
	"-j REJECT --reject-with icmp-proto-unreachable"
)

chain=reductor_maintain
forward=FORWARD

lock() {
	if [ "${filter['maintainance']}" = '0' ] && ! $BINDIR/dnsenabled --revisor; then
		$SETCOLOR_FAILURE
		log "НЕ включаем режим обслуживания, так как он выключен в меню."
		log "В течении этого момента, если происходит обновление или перезапуск Carbon Reductor могут быть пропуски фильтрации."
		$SETCOLOR_NORMAL
		return 0
	fi

	if $BINDIR/dnsenabled --revisor; then
		log "Полная блокировка доступа в интернет для ${filter['revisor_ip']} на время обслуживания."
	else
		$SETCOLOR_FAILURE
		log "Полная блокировка доступа в интернет на время обслуживания."
		$SETCOLOR_NORMAL
	fi
	iptables -N $chain &>/dev/null || true
	iptables -F $chain # because ip can be added or changed
	for rule in "${rules[@]}"; do
		user=""
		if $BINDIR/dnsenabled --revisor; then
			user="-s ${filter['revisor_ip']}"
		fi
		iptables -A $chain $user $rule
	done
	iptables -I $forward $user -j $chain
}

unlock() {
	local user=""
	is_locked || return 0
	if $BINDIR/dnsenabled --revisor; then
		log "Снятие полной блокировки для ${filter['revisor_ip']}, обслуживание сервера завершено."
	else
		log "Снятие полной блокировки, обслуживание сервера завершено."
	fi
	iptables -D $forward $user -j $chain &>/dev/null || true
	if $BINDIR/dnsenabled --revisor; then
		user="-s ${filter['revisor_ip']}"
		iptables -D $forward $user -j $chain &>/dev/null || true
	fi
	for _ in {1..3}; do # Заботимся о пользователях, баги не должны оставить их без инета
		for rule in "${rules[@]}"; do
			user=""
			iptables -D $chain $user $rule &>/dev/null || true
			if $BINDIR/dnsenabled --revisor; then
				user="-s ${filter['revisor_ip']}"
				iptables -D $chain $user $rule &>/dev/null || true
			fi
		done
	done
}

is_locked() {
	iptables -C $forward -j $chain &>/dev/null && log "В текущий момент проводится техническое обслуживание сервера, интернет заблокирован всем." && return 0
	if ! $BINDIR/dnsenabled --revisor; then
		log "В текущий момент технического обслуживания не производится."
		return 1
	fi
	if iptables -C $forward -s ${filter['revisor_ip']} -j $chain &>/dev/null; then
		log "В текущий момент производится техническое обслуживание, интернет заблокирован только для ${filter['revisor_ip']}"
		return 0
	else
		log "В текущий момент технического обслуживания не производится."
		return 1
	fi
}

unlock_delayed() {
	local warning="Система находится на обслуживании. Если это продлится ещё 30 секунд, блокировка будет снята."
	local alarm="Снимаю полную блокировку, несмотря на то, что обслуживание ещё не закончилось. Скорее всего произошло зависание. Нужно разобраться с Carbon Reductor."
	local revisor_locked="Так как указан IP ревизора - не снимаем эту опасную блокировку."
       	if is_locked; then
		log "$warning"
		sleep 30
		if is_locked; then
			if $BINDIR/dnsenabled --revisor; then
				alarm="$revisor_locked"
			fi
			log "$alarm"
			log "Администраторам будет послано уведомление об этом."
			echo "$alarm" > $TMPDIR/alarm.unlock
			send_alarm $TMPDIR/alarm.unlock
			if ! $BINDIR/dnsenabled --revisor; then
				unlock
			fi
		fi
	fi
}

main() {
	case "$1" in
		lock | unlock | unlock_delayed | is_locked )
			"$1"
			;;
		* )
			log "Вызвана неизвестная команда: $*"
			exit 1
	esac
}

main "$@"
